"""OpenCode backend for session explorer.

OpenCode is an open-source AI coding agent. This backend provides support for
viewing and interacting with OpenCode sessions through the session explorer.
"""

from .backend import OpenCodeBackend

__all__ = ["OpenCodeBackend"]
