<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no, viewport-fit=cover">
    <title>{% block title %}Claude Code transcript{% endblock %}</title>
    <style>{{ css|safe }}</style>
</head>
<body>
    <div class="container">
{%- block content %}{% endblock %}
    </div>
    <script>{{ js|safe }}</script>
</body>
</html>
